/*
 * AutomatonIterationEpsilonTransition.cpp
 *
 *  Created on: 20. 11. 2014
 *	  Author: Tomas Pecka
 */

#include "AutomatonIterationEpsilonTransition.h"
#include <common/createUnique.hpp>

namespace automaton {

namespace transform {

automaton::Automaton AutomatonIterationEpsilonTransition::iteration(const automaton::Automaton& automaton) {
	return dispatch(automaton.getData());
}

template<class T>
automaton::EpsilonNFA < > AutomatonIterationEpsilonTransition::iteration(const T& automaton) {
	automaton::EpsilonNFA < > res(automaton);

	for(const auto&q : automaton.getFinalStates())
		res.addTransition(q, automaton.getInitialState());

	label::Label newInitialState = common::createUnique(automaton.getInitialState(), automaton.getStates());
	res.addState(newInitialState);
	res.setInitialState(newInitialState);
	res.addFinalState(newInitialState);

	res.addTransition(newInitialState, automaton.getInitialState());
	return res;
}

auto AutomatonIterationEpsilonTransitionDFA = AutomatonIterationEpsilonTransition::RegistratorWrapper<automaton::EpsilonNFA < >, automaton::DFA<>>(AutomatonIterationEpsilonTransition::iteration);
auto AutomatonIterationEpsilonTransitionNFA = AutomatonIterationEpsilonTransition::RegistratorWrapper<automaton::EpsilonNFA < >, automaton::NFA < > >(AutomatonIterationEpsilonTransition::iteration);
auto AutomatonIterationEpsilonTransitionEpsilonNFA = AutomatonIterationEpsilonTransition::RegistratorWrapper<automaton::EpsilonNFA < >, automaton::EpsilonNFA < > >(AutomatonIterationEpsilonTransition::iteration);

} /* namespace transform */

} /* namespace automaton */
