/*
 * GlushkovFollow.cpp
 *
 *  Created on: 14. 4. 2016
 *	  Author: Tomas Pecka
 */

#include "GlushkovFollow.h"
#include "GlushkovFirst.h"
#include "GlushkovPos.h"
#include <iterator>
#include <vector>

namespace rte {

std::set < std::vector < std::ranked_symbol < > > > GlushkovFollow::follow ( const rte::FormalRTE < > & rte, const std::ranked_symbol < > & symbol ) {
	std::map < std::ranked_symbol < >, std::set < std::ranked_symbol < > > > subMap;

	 /* Init substitution map, ie \forall a \in K: sub[a] = \emptyset */
	for ( const std::ranked_symbol < > & ssymb : rte.getSubstitutionAlphabet ( ) )
		subMap.insert ( std::make_pair ( ssymb, std::set < std::ranked_symbol < > > { } ) );

	 /* recursively compute follow */
	return rte.getRTE ( ).getStructure ( ).accept < std::set < std::vector < std::ranked_symbol < > > >, GlushkovFollow::Formal > ( symbol, rte.getSubstitutionAlphabet ( ), subMap );
}

// -----------------------------------------------------------------------------

template < class T >
void cartesian_rec ( const std::vector < std::vector < T > > & input, std::vector < std::vector < T > > & ret, std::vector < T > & current, size_t depth ) {
	if ( depth == input.size ( ) )
		ret.push_back ( current );
	else
		for ( size_t i = 0; i < input[depth].size ( ); i++ ) {
			current[depth] = input[depth][i];
			cartesian_rec ( input, ret, current, depth + 1 );
		}

}

template < class T >
std::vector < std::vector < T > > cartesian ( const std::vector < std::vector < T > > & input ) {
	std::vector < std::vector < T > > ret;
	std::vector < T > current ( input.size ( ), std::ranked_symbol < > ( 0, 0 ) );

	cartesian_rec ( input, ret, current, 0 );

	return ret;
}

void preprocessSubMap ( const std::set < std::ranked_symbol < > > & alphabetK, std::map < std::ranked_symbol < >, std::set < std::ranked_symbol < > > > & subMap ) {
	for ( bool change = true; change; change = false )
		for ( std::pair < const std::ranked_symbol < >, std::set < std::ranked_symbol < > > > & kv : subMap ) {
			std::set < std::ranked_symbol < > > & substSet = kv.second;

			for ( const auto & e : substSet ) {
				if ( alphabetK.count ( e ) == 0 ) continue;

				auto it = subMap.find ( e );
				size_t oldSize = substSet.size ( );
				substSet.insert ( it->second.begin ( ), it->second.end ( ) );
				change = ( oldSize != substSet.size ( ) ); // something was added
				substSet.erase ( e );
			}
		}
}

std::set < std::vector < std::ranked_symbol < > > > replaceConstants ( const std::set < std::ranked_symbol < > > & alphabetK, const std::vector < std::ranked_symbol < > > & follow, const std::map < std::ranked_symbol < >, std::set < std::ranked_symbol < > > > & subMap2 ) {
	std::map < std::ranked_symbol < >, std::set < std::ranked_symbol < > > > subMap ( subMap2 );
	preprocessSubMap ( alphabetK, subMap );

	std::vector < std::vector < std::ranked_symbol < > > > input;

	for ( const std::ranked_symbol < > & e : follow ) {
		if ( alphabetK.count ( e ) > 0 )
			input.push_back ( std::vector < std::ranked_symbol < > > ( subMap.at ( e ).begin ( ), subMap.at ( e ).end ( ) ) );
		else
			input.push_back ( std::vector < std::ranked_symbol < > > { e } );
	}

	 /* now do the cartesian product on "input" */
	std::vector < std::vector < std::ranked_symbol < > > > followSet = cartesian ( input );
	return std::set < std::vector < std::ranked_symbol < > > > ( followSet.begin ( ), followSet.end ( ) );
}

// -----------------------------------------------------------------------------

std::set < std::vector < std::ranked_symbol < > > > GlushkovFollow::Formal::visit ( const rte::FormalRTEAlternation < alphabet::Symbol, primitive::Unsigned > & node, const std::ranked_symbol < > & symbolF, const std::set < std::ranked_symbol < > > & alphabetK, std::map < std::ranked_symbol < >, std::set < std::ranked_symbol < > > > & subMap ) {
	std::set < std::vector < std::ranked_symbol < > > > ret, tmp;

	tmp = node.getLeftElement ( ).accept < std::set < std::vector < std::ranked_symbol < > > >, GlushkovFollow::Formal > ( symbolF, alphabetK, subMap );
	ret.insert ( tmp.begin ( ), tmp.end ( ) );

	tmp = node.getRightElement ( ).accept < std::set < std::vector < std::ranked_symbol < > > >, GlushkovFollow::Formal > ( symbolF, alphabetK, subMap );
	ret.insert ( tmp.begin ( ), tmp.end ( ) );

	return ret;
}

std::set < std::vector < std::ranked_symbol < > > > GlushkovFollow::Formal::visit ( const rte::FormalRTESubstitution < alphabet::Symbol, primitive::Unsigned > & node, const std::ranked_symbol < > & symbolF, const std::set < std::ranked_symbol < > > & alphabetK, std::map < std::ranked_symbol < >, std::set < std::ranked_symbol < > > > & subMap ) {

	std::map < std::ranked_symbol < >, std::set < std::ranked_symbol < > > > subMap2 ( subMap );
	auto itMap = subMap2.find ( node.getSubstitutionSymbol ( ).getSymbol ( ) );

	itMap->second.clear ( );

	for ( const auto & s : node.getRightElement ( ).accept < std::set < std::ranked_symbol < > >, GlushkovFirst::Formal > ( ) )
		itMap->second.insert ( s );

	/*
	 * E sub F
	 *   1. if symbolF in F subtree, then Follow(F, symbolF);
	 *   2. if symbolF in E subtree, then Follow(E, symbolF);
	 */

	if ( node.getLeftElement ( ).accept < bool, GlushkovPos::Formal > ( symbolF ) )
		return node.getLeftElement ( ).accept < std::set < std::vector < std::ranked_symbol < > > >, GlushkovFollow::Formal > ( symbolF, alphabetK, subMap2 );
	else
		return node.getRightElement ( ).accept < std::set < std::vector < std::ranked_symbol < > > >, GlushkovFollow::Formal > ( symbolF, alphabetK, subMap );
}

std::set < std::vector < std::ranked_symbol < > > > GlushkovFollow::Formal::visit ( const rte::FormalRTEIteration < alphabet::Symbol, primitive::Unsigned > & node, const std::ranked_symbol < > & symbolF, const std::set < std::ranked_symbol < > > & alphabetK, std::map < std::ranked_symbol < >, std::set < std::ranked_symbol < > > > & subMap ) {

	std::set < std::vector < std::ranked_symbol < > > > ret;

	for ( const auto & s : node.getElement ( ).accept < std::set < std::ranked_symbol < > >, GlushkovFirst::Formal > ( ) )
		subMap[node.getSubstitutionSymbol ( ).getSymbol ( )].insert ( s );

	return node.getElement ( ).accept < std::set < std::vector < std::ranked_symbol < > > >, GlushkovFollow::Formal > ( symbolF, alphabetK, subMap );
}

std::set < std::vector < std::ranked_symbol < > > > GlushkovFollow::Formal::visit ( const rte::FormalRTESymbolAlphabet < alphabet::Symbol, primitive::Unsigned > & node, const std::ranked_symbol < > & symbolF, const std::set < std::ranked_symbol < > > & alphabetK, std::map < std::ranked_symbol < >, std::set < std::ranked_symbol < > > > & subMap ) {

	std::set < std::vector < std::ranked_symbol < > > > ret, tmp;

	if ( symbolF == node.getSymbol ( ) ) {
		std::vector < std::ranked_symbol < > > children;

		for ( const std::smart_ptr < const rte::FormalRTESymbol < alphabet::Symbol, primitive::Unsigned > > & c : node.getElements ( ) )
			children.push_back ( c->getSymbol ( ) );

		return replaceConstants ( alphabetK, children, subMap );
	}

	for ( const auto & c : node.getElements ( ) ) {
		tmp = c->accept < std::set < std::vector < std::ranked_symbol < > > >, GlushkovFollow::Formal > ( symbolF, alphabetK, subMap );
		ret.insert ( tmp.begin ( ), tmp.end ( ) );
	}

	return ret;
}

std::set < std::vector < std::ranked_symbol < > > > GlushkovFollow::Formal::visit ( const rte::FormalRTESymbolSubst < alphabet::Symbol, primitive::Unsigned > & /* node */, const std::ranked_symbol < > & /* symbolF */, const std::set < std::ranked_symbol < > > & /* alphabetK */, std::map < std::ranked_symbol < >, std::set < std::ranked_symbol < > > > & /* subMap */ ) {
	return std::set < std::vector < std::ranked_symbol < > > > ( );
}

std::set < std::vector < std::ranked_symbol < > > > GlushkovFollow::Formal::visit ( const rte::FormalRTEEmpty < alphabet::Symbol, primitive::Unsigned > & /* node */, const std::ranked_symbol < > & /* symbolF */, const std::set < std::ranked_symbol < > > & /* alphabetK */, std::map < std::ranked_symbol < >, std::set < std::ranked_symbol < > > > & /* subMap */ ) {
	return std::set < std::vector < std::ranked_symbol < > > > ( );
}

} /* namespace rte */
