/*
 * GlushkovFollow.h
 *
 *  Created on: 14. 4. 2016
 *	  Author: Tomas Pecka
 */

#ifndef RTE_GLUSHKOV_FOLLOW_H_
#define RTE_GLUSHKOV_FOLLOW_H_

#include <vector>
#include <map>
#include <set>

#include <rte/formal/FormalRTE.h>
#include <rte/formal/FormalRTEElements.h>

#include <alphabet/RankedSymbol.h>

namespace rte {

class GlushkovFollow {
public:
	/**
	 * @param re rte to probe
	 * @param symbol FormalRTESymbol for which we need the follow(), i.e., Follow(RTE, symbol)
	 * @return all symbols that can follow specific symbol in word
	 */
	static std::set < std::vector < std::ranked_symbol < > > > follow ( const rte::FormalRTE < > & re, const std::ranked_symbol < > & symbol );

	class Formal {
	public:
		static std::set < std::vector < std::ranked_symbol < > > > visit ( const rte::FormalRTEElement < alphabet::Symbol, primitive::Unsigned > & node, const std::ranked_symbol < > & symbol, const std::set < std::ranked_symbol < > > & alphabetK, std::map < std::ranked_symbol < >, std::set < std::ranked_symbol < > > > & subM );
		static std::set < std::vector < std::ranked_symbol < > > > visit ( const rte::FormalRTEAlternation < alphabet::Symbol, primitive::Unsigned > & node, const std::ranked_symbol < > & symbol, const std::set < std::ranked_symbol < > > & alphabetK, std::map < std::ranked_symbol < >, std::set < std::ranked_symbol < > > > & subM );
		static std::set < std::vector < std::ranked_symbol < > > > visit ( const rte::FormalRTESubstitution < alphabet::Symbol, primitive::Unsigned > & node, const std::ranked_symbol < > & symbol, const std::set < std::ranked_symbol < > > & alphabetK, std::map < std::ranked_symbol < >, std::set < std::ranked_symbol < > > > & subM );
		static std::set < std::vector < std::ranked_symbol < > > > visit ( const rte::FormalRTEIteration < alphabet::Symbol, primitive::Unsigned > & node, const std::ranked_symbol < > & symbol, const std::set < std::ranked_symbol < > > & alphabetK, std::map < std::ranked_symbol < >, std::set < std::ranked_symbol < > > > & subM );
		static std::set < std::vector < std::ranked_symbol < > > > visit ( const rte::FormalRTESymbolAlphabet < alphabet::Symbol, primitive::Unsigned > & node, const std::ranked_symbol < > & symbol, const std::set < std::ranked_symbol < > > & alphabetK, std::map < std::ranked_symbol < >, std::set < std::ranked_symbol < > > > & subM );
		static std::set < std::vector < std::ranked_symbol < > > > visit ( const rte::FormalRTESymbolSubst < alphabet::Symbol, primitive::Unsigned > & node, const std::ranked_symbol < > & symbol, const std::set < std::ranked_symbol < > > & alphabetK, std::map < std::ranked_symbol < >, std::set < std::ranked_symbol < > > > & subM );
		static std::set < std::vector < std::ranked_symbol < > > > visit ( const rte::FormalRTEEmpty < alphabet::Symbol, primitive::Unsigned > & node, const std::ranked_symbol < > & symbol, const std::set < std::ranked_symbol < > > & alphabetK, std::map < std::ranked_symbol < >, std::set < std::ranked_symbol < > > > & subM );
	};
};

} /* namespace rte */

#endif /* RTE_GLUSHKOV_FOLLOW_H_ */
