/*
 * ExactMatchingAutomaton.cpp
 *
 *  Created on: 9. 2. 2014
 *      Author: Jan Travnicek
 */

#include "ExactMatchingAutomaton.h"
#include <string/LinearString.h>
#include <string/Epsilon.h>

#include <deque>

namespace stringology {

namespace exact {

automaton::Automaton ExactMatchingAutomaton::construct(const string::String& pattern) {
	return dispatch(pattern.getData());
}

automaton::NFA < > ExactMatchingAutomaton::construct(const string::LinearString < >& pattern) {
	automaton::NFA < > res(label::Label(0));
	res.setInputAlphabet(pattern.getAlphabet());
	for(const alphabet::Symbol& symbol : pattern.getAlphabet()) {
		res.addTransition(label::Label(0), symbol, label::Label(0));
	}
	int i = 1;
	for(const alphabet::Symbol& symbol : pattern.getContent()) {
		res.addState(label::Label(i));
		res.addTransition(label::Label(i-1), symbol, label::Label(i));
		i++;
	}
	res.addFinalState(label::Label(i-1));
	return res;
}

auto ExactMatchingAutomatonLinearString = ExactMatchingAutomaton::RegistratorWrapper<automaton::NFA < > , string::LinearString < >>(ExactMatchingAutomaton::construct);

} /* namespace exact */

} /* namespace stringology */
